Attribute VB_Name = "SHfileOps"
'SHfileOps.bas
'-------------
'Declarations for and sample functions using the Win32 SHFileOperation API
'=========================================================================
'Using the SHFileOperation API to manipulate files displays the
'Explorer-type dialogs and progress / animation effects.

'DISCLAIMER - This code was hacked out (is there any other way?)
'and tested for -well not so much tested as "Hey, IT WORKS!".
'So here it is. (Experimentation is the key to enlightenment.)
'Additional comments, corrections, etc are encouraged.

'NOTES:
'*Always include the full Path\filename to avoid unexpected results.
'(see implicit parameters below.)

'Single filenames:
'Filenames passed to SHFileOperation must be null-terminated.
'example:   "C:\vbasic\myfile" & chr(0)

'Multiple filenames:
'Multiple filenames can be passed for copy, delete, and move.
'SHFileOperation requires that multiple filenames passed in
'a single VB string be null-delimited and null-terminated.
'example:  filename1+Chr(0)+filename2+Chr(0)+filename3+Chr(0)
'**Include the full path in the filenames**.
'-------------------------------------------------------------
'This is *NOT* the same format as returned by the CommonDialog
'control when the cdlOFNAllowMultiselect flag is set.
'The CommonDialog format for multiple selections is:
'   Path+Chr(0)+filename1+Chr(0)+filename2+Chr(0)+Chr(0)
'(The path only appears once at the start of the list and
'is not included with each filename. The list is terminated
'with a double-null.)



Option Explicit

'  Shell File Operations
Const FO_MOVE = &H1
Const FO_COPY = &H2
Const FO_DELETE = &H3
Const FO_RENAME = &H4
'--Flags--------------------------
'Const FOF_MULTIDESTFILES = &H1
Const FOF_CONFIRMMOUSE = &H2
'Const FOF_SILENT = &H4              '  don't create progress/report
'Const FOF_RENAMEONCOLLISION = &H8
Const FOF_NOCONFIRMATION = &H10     '  Don't prompt the user.
'Const FOF_WANTMAPPINGHANDLE = &H20  '  Fill in SHFILEOPSTRUCT.hNameMappings
                                    '  Must be freed using SHFreeNameMappings
Const FOF_ALLOWUNDO = &H40          '  send to RECYCLE BIN on delete
Const FOF_FILESONLY = &H80          '  on *.*, do only files, not dirs
Const FOF_SIMPLEPROGRESS = &H100    '  means don't show names of files
Const FOF_NOCONFIRMMKDIR = &H200    '  don't confirm making any needed dirs


'  **implicit parameters are:
'       if pFrom or pTo are unqualified names the current directories are
'       taken from the global current drive/directory settings managed
'       by Get/SetCurrentDrive/Directory


Private Type SHFILEOPSTRUCT
        hwnd As Long            'hWnd of owner (calling form)
        wFunc As Long           'a single(1) FO_ constant
        pFrom As String         'source filename(s)
        pTo As String           'destination (ignored if FO_DELETE)
        fFlags As Integer       'combination of FOF_ flags
        fAnyOperationsAborted As Long  '??
        hNameMappings As Long       '??
        lpszProgressTitle As Long '  only used if FOF_SIMPLEPROGRESS
End Type

'Private Declare Sub SHFreeNameMappings Lib "shell32.dll" (ByVal hNameMappings As Long)

'Private Type SHNAMEMAPPING
'        pszOldPath As String
'        pszNewPath As String
'        cchOldPath As Long
'        cchNewPath As Long
'End Type

Private Declare Function SHFileOperation Lib "shell32.dll" Alias "SHFileOperationA" _
(lpFileOp As SHFILEOPSTRUCT) As Long


'SAMPLE FUNCTIONS USING SHFileOperation
'======================================

Public Function ShellCopy(hOwner As Long, sFrom As String, sTo As String, Optional bConfirm) As Long
Dim shOP As SHFILEOPSTRUCT

shOP.hwnd = hOwner
shOP.wFunc = FO_COPY
shOP.pFrom = sFrom & Chr(0)
shOP.pTo = sTo & Chr(0)
shOP.fFlags = FOF_CONFIRMMOUSE

'default for bConfirm =True
If Not IsMissing(bConfirm) Then
    If bConfirm = False Then
        shOP.fFlags = shOP.fFlags Or FOF_NOCONFIRMATION
    End If
End If

ShellCopy = SHFileOperation(shOP)
End Function

Public Function ShellDelete(hOwner As Long, sFileName, Optional bConfirm) As Long
'Nukes the file (bypasses Recycle Bin)
Dim shOP As SHFILEOPSTRUCT

shOP.hwnd = hOwner
shOP.wFunc = FO_DELETE
shOP.pFrom = sFileName & Chr(0)
shOP.fFlags = FOF_CONFIRMMOUSE

'default for bConfirm =True
If Not IsMissing(bConfirm) Then
    If bConfirm = False Then
        shOP.fFlags = shOP.fFlags Or FOF_NOCONFIRMATION
    End If
End If

ShellDelete = SHFileOperation(shOP)
End Function

Public Function ShellMove(hOwner As Long, sFrom As String, sTo As String, Optional bConfirm) As Long
Dim shOP As SHFILEOPSTRUCT

shOP.hwnd = hOwner
shOP.wFunc = FO_MOVE
shOP.pFrom = sFrom & Chr(0)
shOP.pTo = sTo & Chr(0)
shOP.fFlags = FOF_CONFIRMMOUSE

'default for bConfirm =True
If Not IsMissing(bConfirm) Then
    If bConfirm = False Then
        shOP.fFlags = shOP.fFlags Or FOF_NOCONFIRMATION
    End If
End If

ShellMove = SHFileOperation(shOP)
End Function

Public Function ShellReName(hOwner As Long, sFrom As String, sTo As String, Optional bConfirm) As Long
'Can only change filename (not path)
'Path portions of sFrom and sTo must be identical
Dim shOP As SHFILEOPSTRUCT

shOP.hwnd = hOwner
shOP.wFunc = FO_RENAME
shOP.pFrom = sFrom & Chr(0)
shOP.pTo = sTo & Chr(0)
shOP.fFlags = FOF_CONFIRMMOUSE

'default for bConfirm =True
If Not IsMissing(bConfirm) Then
    If bConfirm = False Then
        shOP.fFlags = shOP.fFlags Or FOF_NOCONFIRMATION
    End If
End If

ShellReName = SHFileOperation(shOP)
End Function

Public Function ShellSendToRecycle(hOwner As Long, sFileName, Optional bConfirm) As Long
'This code is identical to ShellDelete except for FOF_ALLOWUNDO flag.
'Does a permanent delete if no Recycle Bin is defined for the drive.
Dim shOP As SHFILEOPSTRUCT

shOP.hwnd = hOwner
shOP.wFunc = FO_DELETE
shOP.pFrom = sFileName & Chr(0)
shOP.fFlags = FOF_CONFIRMMOUSE Or FOF_ALLOWUNDO

'default for bConfirm =True
If Not IsMissing(bConfirm) Then
    If bConfirm = False Then
        shOP.fFlags = shOP.fFlags Or FOF_NOCONFIRMATION
    End If
End If

ShellSendToRecycle = SHFileOperation(shOP)
End Function

